@@ -193,6 +193,12 @@ type ProgramTestOptions struct {
 	// SkipStackRemoval indicates that the stack should not be removed. (And so the test's results could be inspected
 	// in the Pulumi Service after the test has completed.)
 	SkipStackRemoval bool
+	// Destroy on cleanup defers stack destruction until the test cleanup step, rather than after
+	// program test execution. This is useful for more realistic stack reference testing, allowing one
+	// project and stack to be stood up and a second to be run before the first is destroyed.
+	//
+	// Implies NoParallel because we expect that another caller to ProgramTest will set that
+	DestroyOnCleanup bool
 	// Quick implies SkipPreview, SkipExportImport and SkipEmptyPreviewUpdate
 	Quick bool
 	// PreviewCommandlineFlags specifies flags to add to the `pulumi preview` command line (e.g. "--color=raw")
@@ -291,6 +297,12 @@ type ProgramTestOptions struct {
 	// If set, this hook is called after `pulumi stack export` on the exported file. If `SkipExportImport` is set, this
 	// hook is ignored.
 	ExportStateValidator func(t *testing.T, stack []byte)
+
+	// If not nil, specifies the logic of preparing a project by
+	// ensuring dependencies. If left as nil, runs default
+	// preparation logic by dispatching on whether the project
+	// uses Node, Python, .NET or Go.
+	PrepareProject func(*engine.Projinfo) error
 }
 
 func (opts *ProgramTestOptions) GetDebugLogLevel() int {
@@ -427,6 +439,9 @@ func (opts ProgramTestOptions) With(overrides ProgramTestOptions) ProgramTestOpt
 	if overrides.SkipStackRemoval {
 		opts.SkipStackRemoval = overrides.SkipStackRemoval
 	}
+	if overrides.DestroyOnCleanup {
+		opts.DestroyOnCleanup = overrides.DestroyOnCleanup
+	}
 	if overrides.Quick {
 		opts.Quick = overrides.Quick
 	}
@@ -502,6 +517,9 @@ func (opts ProgramTestOptions) With(overrides ProgramTestOptions) ProgramTestOpt
 	if overrides.UsePipenv {
 		opts.UsePipenv = overrides.UsePipenv
 	}
+	if overrides.PrepareProject != nil {
+		opts.PrepareProject = overrides.PrepareProject
+	}
 	return opts
 }
 
@@ -586,7 +604,7 @@ func prepareProgram(t *testing.T, opts *ProgramTestOptions) {
 	}
 
 	// We want tests to default into being ran in parallel, hence the odd double negative.
-	if !opts.NoParallel {
+	if !opts.NoParallel && !opts.DestroyOnCleanup {
 		t.Parallel()
 	}
 
@@ -1037,18 +1055,28 @@ func (pt *ProgramTester) TestLifeCycleInitAndDestroy() error {
 	}
 
 	pt.TestFinished = false
-	defer pt.TestCleanUp()
+	if pt.opts.DestroyOnCleanup {
+		pt.t.Cleanup(pt.TestCleanUp)
+	} else {
+		defer pt.TestCleanUp()
+	}
 
 	err = pt.TestLifeCycleInitialize()
 	if err != nil {
 		return fmt.Errorf("initializing test project: %w", err)
 	}
 
-	// Ensure that before we exit, we attempt to destroy and remove the stack.
-	defer func() {
+	destroyStack := func() {
 		destroyErr := pt.TestLifeCycleDestroy()
 		assert.NoError(pt.t, destroyErr)
-	}()
+	}
+	if pt.opts.DestroyOnCleanup {
+		// Allow other tests to refer to this stack until the test is complete.
+		pt.t.Cleanup(destroyStack)
+	} else {
+		// Ensure that before we exit, we attempt to destroy and remove the stack.
+		defer destroyStack()
+	}
 
 	if err = pt.TestPreviewUpdateAndEdits(); err != nil {
 		return fmt.Errorf("running test preview, update, and edits: %w", err)
@@ -1685,19 +1713,10 @@ func (pt *ProgramTester) getProjinfo(projectDir string) (*engine.Projinfo, error
 
 // prepareProject runs setup necessary to get the project ready for `pulumi` commands.
 func (pt *ProgramTester) prepareProject(projinfo *engine.Projinfo) error {
-	// Based on the language, invoke the right routine to prepare the target directory.
-	switch rt := projinfo.Proj.Runtime.Name(); rt {
-	case NodeJSRuntime:
-		return pt.prepareNodeJSProject(projinfo)
-	case PythonRuntime:
-		return pt.preparePythonProject(projinfo)
-	case GoRuntime:
-		return pt.prepareGoProject(projinfo)
-	case DotNetRuntime:
-		return pt.prepareDotNetProject(projinfo)
-	default:
-		return fmt.Errorf("unrecognized project runtime: %s", rt)
+	if pt.opts.PrepareProject != nil {
+		return pt.opts.PrepareProject(projinfo)
 	}
+	return pt.defaultPrepareProject(projinfo)
 }
 
 // prepareProjectDir runs setup necessary to get the project ready for `pulumi` commands.
@@ -2090,3 +2109,19 @@ func (pt *ProgramTester) prepareDotNetProject(projinfo *engine.Projinfo) error {
 
 	return nil
 }
+
+func (pt *ProgramTester) defaultPrepareProject(projinfo *engine.Projinfo) error {
+	// Based on the language, invoke the right routine to prepare the target directory.
+	switch rt := projinfo.Proj.Runtime.Name(); rt {
+	case NodeJSRuntime:
+		return pt.prepareNodeJSProject(projinfo)
+	case PythonRuntime:
+		return pt.preparePythonProject(projinfo)
+	case GoRuntime:
+		return pt.prepareGoProject(projinfo)
+	case DotNetRuntime:
+		return pt.prepareDotNetProject(projinfo)
+	default:
+		return fmt.Errorf("unrecognized project runtime: %s", rt)
+	}
+}
